(function () {

/* Imports */
var Meteor = Package.meteor.Meteor;
var SandstormDb = Package['sandstorm-db'].SandstormDb;

/* Package-scope variables */
var SandstormPermissions, identityId;

(function () {

////////////////////////////////////////////////////////////////////////////////////////////////////////
//                                                                                                    //
// packages/sandstorm-permissions/permissions.js                                                      //
//                                                                                                    //
////////////////////////////////////////////////////////////////////////////////////////////////////////
                                                                                                      //
// Sandstorm - Personal Cloud Sandbox                                                                 // 1
// Copyright (c) 2015 Sandstorm Development Group, Inc. and contributors                              // 2
// All rights reserved.                                                                               // 3
//                                                                                                    // 4
// Licensed under the Apache License, Version 2.0 (the "License");                                    // 5
// you may not use this file except in compliance with the License.                                   // 6
// You may obtain a copy of the License at                                                            // 7
//                                                                                                    // 8
//   http://www.apache.org/licenses/LICENSE-2.0                                                       // 9
//                                                                                                    // 10
// Unless required by applicable law or agreed to in writing, software                                // 11
// distributed under the License is distributed on an "AS IS" BASIS,                                  // 12
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.                           // 13
// See the License for the specific language governing permissions and                                // 14
// limitations under the License.                                                                     // 15
                                                                                                      // 16
var Crypto = Npm.require("crypto");                                                                   // 17
var Url = Npm.require("url");                                                                         // 18
                                                                                                      // 19
SandstormPermissions = {};                                                                            // 20
                                                                                                      // 21
function PermissionSet(array) {                                                                       // 22
  // A wrapper around an array of booleans.                                                           // 23
                                                                                                      // 24
  if (!array) {                                                                                       // 25
    this.array = [];                                                                                  // 26
  } else if (array instanceof Array) {                                                                // 27
    this.array = array.slice(0);                                                                      // 28
  } else {                                                                                            // 29
    throw new Error("don't know how to interpret as PermissionSet: " + array);                        // 30
  }                                                                                                   // 31
}                                                                                                     // 32
                                                                                                      // 33
// Methods for mutating a PermissionSet by combining it with another PermissionSet.                   // 34
// These return a boolean indicating whether the operation had any effect.                            // 35
                                                                                                      // 36
PermissionSet.prototype.add = function(other) {                                                       // 37
  var changed = false;                                                                                // 38
  for (var ii = 0; ii < other.array.length; ++ii) {                                                   // 39
    var old = this.array[ii];                                                                         // 40
    this.array[ii] = this.array[ii] || other.array[ii];                                               // 41
    if (old != this.array[ii]) {                                                                      // 42
      changed = true;                                                                                 // 43
    }                                                                                                 // 44
  }                                                                                                   // 45
  return changed;                                                                                     // 46
}                                                                                                     // 47
                                                                                                      // 48
PermissionSet.prototype.remove = function(other) {                                                    // 49
  var changed = false;                                                                                // 50
  for (var ii = 0; ii < other.array.length && ii < this.array.length; ++ii) {                         // 51
    var old = this.array[ii];                                                                         // 52
    this.array[ii] = this.array[ii] && !other.array[ii];                                              // 53
    if (old != this.array[ii]) {                                                                      // 54
      changed = true;                                                                                 // 55
    }                                                                                                 // 56
  }                                                                                                   // 57
  return changed;                                                                                     // 58
}                                                                                                     // 59
                                                                                                      // 60
PermissionSet.prototype.intersect = function(other) {                                                 // 61
  var changed = false;                                                                                // 62
  for (var ii = 0; ii < this.array.length; ++ii) {                                                    // 63
    var old = this.array[ii];                                                                         // 64
    this.array[ii] = this.array[ii] && other.array[ii];                                               // 65
    if (old != this.array[ii]) {                                                                      // 66
      changed = true;                                                                                 // 67
    }                                                                                                 // 68
  }                                                                                                   // 69
  return changed;                                                                                     // 70
}                                                                                                     // 71
                                                                                                      // 72
function roleAssignmentPermissions(roleAssignment, viewInfo) {                                        // 73
  var result = new PermissionSet([]);                                                                 // 74
                                                                                                      // 75
  if (!roleAssignment || "none" in roleAssignment) {                                                  // 76
    if (viewInfo.roles) {                                                                             // 77
      for (var ii = 0; ii < viewInfo.roles.length; ++ii) {                                            // 78
        var roleDef = viewInfo.roles[ii];                                                             // 79
        if (roleDef.default) {                                                                        // 80
          result = new PermissionSet(roleDef.permissions);                                            // 81
          break;                                                                                      // 82
        }                                                                                             // 83
      }                                                                                               // 84
    }                                                                                                 // 85
  } else if ("allAccess" in roleAssignment) {                                                         // 86
    var length = 0;                                                                                   // 87
    if (viewInfo.permissions) {                                                                       // 88
      length = viewInfo.permissions.length;                                                           // 89
    }                                                                                                 // 90
    var array = new Array(length);                                                                    // 91
    for (var ii = 0; ii < array.length; ++ii) {                                                       // 92
      array[ii] = true;                                                                               // 93
    }                                                                                                 // 94
    result = new PermissionSet(array);                                                                // 95
  } else if ("roleId" in roleAssignment && viewInfo.roles && viewInfo.roles.length > 0) {             // 96
    var roleDef = viewInfo.roles[roleAssignment.roleId];                                              // 97
    if (roleDef) {                                                                                    // 98
      result = new PermissionSet(roleDef.permissions);                                                // 99
    }                                                                                                 // 100
  }                                                                                                   // 101
                                                                                                      // 102
  if (roleAssignment) {                                                                               // 103
    result.add(new PermissionSet(roleAssignment.addPermissionSet));                                   // 104
    result.remove(new PermissionSet(roleAssignment.removePermissionSet));                             // 105
  }                                                                                                   // 106
  return result;                                                                                      // 107
}                                                                                                     // 108
                                                                                                      // 109
function collectEdges(db, vertex) {                                                                   // 110
  // Given a vertex in the sharing graph in the format specified by the `check()` invocation below,   // 111
  // collects the data needed for permissions computations pertaining to that vertex. There are four  // 112
  // self-explanatory special cases for the return value. In order of decreasing precedence, they     // 113
  // are: `{grainDoesNotExist: true}`, `{openerIsOwner: true}`, `{grainIsPublic: true}`, and          // 114
  // `{disallowedAnonymousAccess: true}`. In all other cases, this function returns an object         // 115
  // of the form: `{owner: <identityId>, edgesByRecipient: <object>, terminalEdge: Maybe(<object>)}`. // 116
  // The `owner` field indicates the user who owns the grain. The `edgesByRecipient` field is a map   // 117
  // that coalesces chains of `parentToken` UiView tokens into direct user-to-user edges; its keys    // 118
  // are identity IDs of recipient users and its values are lists of "edge" objects of the form       // 119
  // `{sharer: <identityId>, roleAssignments: <list of role assignments>}`.  The role assignments     // 120
  // should be applied in sequence to compute the set of permissions that flow to a recipient from    // 121
  // a sharer. The `terminalEdge` field is an edge object representing the link to `vertex` from      // 122
  // the nearest user in the sharing graph. If `vertex` is already a user, then this edge is          // 123
  // trivial and its `roleAssignments` field is an empty list. If `terminalEdge` is not present,      // 124
  // then there is no such link.                                                                      // 125
  //                                                                                                  // 126
  // TODO(someday): Once UiView tokens can have membrane requirements, we'll need to account for      // 127
  // them in this computation.                                                                        // 128
  check(vertex,                                                                                       // 129
        Match.OneOf({token: Match.ObjectIncluding({_id: String, grainId: String})},                   // 130
                    {grain: Match.ObjectIncluding(                                                    // 131
                      {_id: String,                                                                   // 132
                       identityId: Match.OneOf(String, null, undefined)})}));                         // 133
                                                                                                      // 134
  var grainId;                                                                                        // 135
  if (vertex.token) {                                                                                 // 136
    grainId = vertex.token.grainId;                                                                   // 137
  } else if (vertex.grain) {                                                                          // 138
    grainId = vertex.grain._id;                                                                       // 139
  }                                                                                                   // 140
  var grain = db.getGrain(grainId);                                                                   // 141
  if (!grain) {                                                                                       // 142
    return {grainDoesNotExist: true};                                                                 // 143
  }                                                                                                   // 144
                                                                                                      // 145
  if (vertex.grain && grain.identityId === vertex.grain.identityId) {                                 // 146
    return {openerIsOwner: true};                                                                     // 147
  }                                                                                                   // 148
                                                                                                      // 149
  if (!grain.private) {                                                                               // 150
    return {grainIsPublic: true};                                                                     // 151
  } else if (vertex.grain && !vertex.grain.identityId) {                                              // 152
    return {disallowedAnonymousAccess: true};                                                         // 153
  }                                                                                                   // 154
                                                                                                      // 155
  var result = {edgesByRecipient: {}, owner: grain.identityId};                                       // 156
  var tokensById = {};                                                                                // 157
  db.collections.apiTokens.find({grainId: grainId,                                                    // 158
                                 revoked: {$ne: true}}).forEach(function(token) {                     // 159
    tokensById[token._id] = token;                                                                    // 160
  });                                                                                                 // 161
                                                                                                      // 162
  function computeEdge(token) {                                                                       // 163
    var roleAssignments = [];                                                                         // 164
    var curToken = token;                                                                             // 165
    while (curToken && curToken.parentToken) {                                                        // 166
      // For a `parentToken` provider, no role assignment means don't attenuate.                      // 167
      if (curToken.roleAssignment) {                                                                  // 168
        roleAssignments.push(curToken.roleAssignment);                                                // 169
      }                                                                                               // 170
      curToken = tokensById[curToken.parentToken];                                                    // 171
    }                                                                                                 // 172
    if (curToken && curToken.grainId && curToken.identityId) {                                        // 173
      roleAssignments.push(curToken.roleAssignment);                                                  // 174
      return {sharer: curToken.identityId, roleAssignments: roleAssignments};                         // 175
    } else {                                                                                          // 176
      return;                                                                                         // 177
    }                                                                                                 // 178
  }                                                                                                   // 179
                                                                                                      // 180
  if (vertex.token) {                                                                                 // 181
    result.terminalEdge = computeEdge(tokensById[vertex.token._id]);                                  // 182
  } else if (vertex.grain) {                                                                          // 183
    result.terminalEdge = {sharer: vertex.grain.identityId, roleAssignments: []};                     // 184
  }                                                                                                   // 185
                                                                                                      // 186
  for (var id in tokensById) {                                                                        // 187
    var token = tokensById[id];                                                                       // 188
    if (Match.test(token.owner, {user: Match.Any})) {                                                 // 189
      var edge = computeEdge(token);                                                                  // 190
      if (edge) {                                                                                     // 191
        var recipient = token.owner.user.identityId ;                                                 // 192
        if (!result.edgesByRecipient[recipient]) {                                                    // 193
          result.edgesByRecipient[recipient] = [];                                                    // 194
        }                                                                                             // 195
        result.edgesByRecipient[recipient].push(edge);                                                // 196
      }                                                                                               // 197
    }                                                                                                 // 198
  }                                                                                                   // 199
  return result;                                                                                      // 200
}                                                                                                     // 201
                                                                                                      // 202
SandstormPermissions.grainPermissions = function(db, vertex, viewInfo) {                              // 203
  // Computes the permissions of a vertex. If the vertex is not allowed to open the grain,            // 204
  // returns null. Otherwise, returns an array of bools representing the permissions held.            // 205
  var edges = collectEdges(db, vertex);                                                               // 206
  if (edges.openerIsOwner) {                                                                          // 207
    return roleAssignmentPermissions({allAccess: null}, viewInfo).array;                              // 208
  }                                                                                                   // 209
  if (edges.grainIsPublic) {                                                                          // 210
    // Grains using the old sharing model always share the default role to anyone who has the         // 211
    // grain URL.                                                                                     // 212
    return roleAssignmentPermissions({none: null}, viewInfo).array;                                   // 213
  }                                                                                                   // 214
  if (edges.grainDoesNotExist || !edges.terminalEdge || edges.disallowedAnonymousAccess) {            // 215
    return null;                                                                                      // 216
  }                                                                                                   // 217
                                                                                                      // 218
  var openerIdentityId = edges.terminalEdge.sharer;                                                   // 219
  var edgesByRecipient = edges.edgesByRecipient;                                                      // 220
  var owner = edges.owner;                                                                            // 221
                                                                                                      // 222
  var permissionsMap = {};                                                                            // 223
  // Keeps track of the permissions that the opener receives from each user. The final result of      // 224
  // our computation will be stored in permissionsMap[owner].                                         // 225
                                                                                                      // 226
  var userStack = [openerIdentityId];                                                                 // 227
                                                                                                      // 228
  var openerAttenuation = roleAssignmentPermissions({allAccess: null}, viewInfo);                     // 229
  edges.terminalEdge.roleAssignments.forEach(function(roleAssignment) {                               // 230
    openerAttenuation.intersect(roleAssignmentPermissions(roleAssignment, viewInfo));                 // 231
  });                                                                                                 // 232
  permissionsMap[openerIdentityId] = openerAttenuation;                                               // 233
                                                                                                      // 234
  while (userStack.length > 0) {                                                                      // 235
    var recipient = userStack.pop();                                                                  // 236
    if (edgesByRecipient[recipient]) {                                                                // 237
      edgesByRecipient[recipient].forEach(function (inEdge) {                                         // 238
        var sharer = inEdge.sharer;                                                                   // 239
        var needToPush = false;                                                                       // 240
        if (!permissionsMap[sharer]) {                                                                // 241
          permissionsMap[sharer] = new PermissionSet();                                               // 242
          needToPush = true;                                                                          // 243
        }                                                                                             // 244
                                                                                                      // 245
        var newPermissions = roleAssignmentPermissions({allAccess: null}, viewInfo);                  // 246
        inEdge.roleAssignments.forEach(function(roleAssignment) {                                     // 247
          newPermissions.intersect(roleAssignmentPermissions(roleAssignment, viewInfo));              // 248
        });                                                                                           // 249
        newPermissions.intersect(permissionsMap[recipient]);                                          // 250
                                                                                                      // 251
        // Optimization: we don't care about permissions that we've already proven the opener has.    // 252
        if (permissionsMap[owner]) {                                                                  // 253
          newPermissions.remove(permissionsMap[owner]);                                               // 254
        }                                                                                             // 255
                                                                                                      // 256
        if (permissionsMap[sharer].add(newPermissions)) {                                             // 257
          needToPush = true;                                                                          // 258
        }                                                                                             // 259
        if (needToPush) {                                                                             // 260
          userStack.push(sharer);                                                                     // 261
        }                                                                                             // 262
      });                                                                                             // 263
    }                                                                                                 // 264
  }                                                                                                   // 265
  if (permissionsMap[owner]) {                                                                        // 266
    return permissionsMap[owner].array;                                                               // 267
  } else {                                                                                            // 268
    return null;                                                                                      // 269
  }                                                                                                   // 270
}                                                                                                     // 271
                                                                                                      // 272
SandstormPermissions.mayOpenGrain = function(db, vertex) {                                            // 273
  // Determines whether the vertex is allowed to open the grain by searching depth first              // 274
  // for a path of active role assignments leading from the grain owner to the vertex.                // 275
                                                                                                      // 276
  var edges = collectEdges(db, vertex);                                                               // 277
  if (edges.grainDoesNotExist || edges.disallowedAnonymousAccess) {                                   // 278
    return false;                                                                                     // 279
  }                                                                                                   // 280
  if (edges.openerIsOwner || edges.grainIsPublic) {                                                   // 281
    return true;                                                                                      // 282
  }                                                                                                   // 283
  var edgesByRecipient = edges.edgesByRecipient;                                                      // 284
  var owner = edges.owner;                                                                            // 285
  if (!edges.terminalEdge) { return false; }                                                          // 286
  if (owner == edges.terminalEdge.sharer) {                                                           // 287
    return true;                                                                                      // 288
  }                                                                                                   // 289
                                                                                                      // 290
  var openerIdentityId = edges.terminalEdge.sharer;                                                   // 291
  var stackedUsers = {};                                                                              // 292
  stackedUsers[openerIdentityId] = true;                                                              // 293
  var userStack = [openerIdentityId];                                                                 // 294
                                                                                                      // 295
  while (userStack.length > 0) {                                                                      // 296
    var recipient = userStack.pop();                                                                  // 297
    var edges = edgesByRecipient[recipient];                                                          // 298
    if (edges) {                                                                                      // 299
      for (var ii = 0; ii < edges.length; ++ii) {                                                     // 300
        var inEdge = edges[ii];                                                                       // 301
        var sharer = inEdge.sharer;                                                                   // 302
        if (sharer == owner) {                                                                        // 303
          return true;                                                                                // 304
        }                                                                                             // 305
        if (!stackedUsers[sharer]) {                                                                  // 306
          userStack.push(sharer);                                                                     // 307
          stackedUsers[sharer] = true;                                                                // 308
        }                                                                                             // 309
      }                                                                                               // 310
    }                                                                                                 // 311
  }                                                                                                   // 312
  return false;                                                                                       // 313
}                                                                                                     // 314
                                                                                                      // 315
SandstormPermissions.downstreamTokens = function(db, root) {                                          // 316
  // Computes a list of the UiView tokens that are downstream in the sharing graph from a given       // 317
  // source. The source, `root`, can either be a token or a (grain, user) pair. The exact format      // 318
  // of `root` is specified in the `check()` invocation below.                                        // 319
  //                                                                                                  // 320
  // TODO(someday): Once UiView tokens can have membrane requirements, we'll need to account for      // 321
  // them in this computation.                                                                        // 322
                                                                                                      // 323
  check(root, Match.OneOf({token: Match.ObjectIncluding({_id: String, grainId: String})},             // 324
                          {grain: Match.ObjectIncluding({_id: String, identityId: String})}));        // 325
                                                                                                      // 326
  var result = [];                                                                                    // 327
  var tokenStack = [];                                                                                // 328
  var stackedTokens = {};                                                                             // 329
  var tokensBySharer = {};                                                                            // 330
  var tokensByParent = {};                                                                            // 331
  var tokensById = {};                                                                                // 332
                                                                                                      // 333
  function addChildren(tokenId) {                                                                     // 334
    var children = tokensByParent[tokenId];                                                           // 335
    if (children) {                                                                                   // 336
      children.forEach(function (child) {                                                             // 337
        if (!stackedTokens[child._id]) {                                                              // 338
          tokenStack.push(child);                                                                     // 339
          stackedTokens[child._id] = true;                                                            // 340
        }                                                                                             // 341
      });                                                                                             // 342
    }                                                                                                 // 343
  }                                                                                                   // 344
                                                                                                      // 345
  function addSharedTokens(sharer) {                                                                  // 346
    var sharedTokens = tokensBySharer[sharer];                                                        // 347
    if (sharedTokens) {                                                                               // 348
      sharedTokens.forEach(function (sharedToken) {                                                   // 349
        if (!stackedTokens[sharedToken._id]) {                                                        // 350
          tokenStack.push(sharedToken);                                                               // 351
          stackedTokens[sharedToken._id] = true;                                                      // 352
        }                                                                                             // 353
      });                                                                                             // 354
    }                                                                                                 // 355
  }                                                                                                   // 356
                                                                                                      // 357
  var grainId;                                                                                        // 358
  if (root.token) {                                                                                   // 359
    grainId = root.token.grainId;                                                                     // 360
  } else if (root.grain) {                                                                            // 361
    grainId = root.grain._id;                                                                         // 362
  }                                                                                                   // 363
                                                                                                      // 364
  var grain = db.getGrain(grainId);                                                                   // 365
  if (!grain || !grain.private ) { return result; }                                                   // 366
                                                                                                      // 367
  db.collections.apiTokens.find({grainId: grainId,                                                    // 368
                                 revoked: {$ne: true}}).forEach(function (token) {                    // 369
    tokensById[token._id] = token;                                                                    // 370
    if (token.parentToken) {                                                                          // 371
      if (!tokensByParent[token.parentToken]) {                                                       // 372
        tokensByParent[token.parentToken] = [];                                                       // 373
      }                                                                                               // 374
      tokensByParent[token.parentToken].push(token);                                                  // 375
    } else if (token.identityId) {                                                                    // 376
      if (!tokensBySharer[token.identityId]) {                                                        // 377
        tokensBySharer[token.identityId] = [];                                                        // 378
      }                                                                                               // 379
      tokensBySharer[token.identityId].push(token);                                                   // 380
    }                                                                                                 // 381
  });                                                                                                 // 382
                                                                                                      // 383
  if (root.token) {                                                                                   // 384
    addChildren(root.token._id);                                                                      // 385
  } else if (root.grain) {                                                                            // 386
    addSharedTokens(root.grain.identityId);                                                           // 387
  }                                                                                                   // 388
                                                                                                      // 389
  while (tokenStack.length > 0) {                                                                     // 390
    var token = tokenStack.pop();                                                                     // 391
    result.push(token);                                                                               // 392
    addChildren(token._id);                                                                           // 393
    if (token.owner && token.owner.user) {                                                            // 394
      addSharedTokens(token.owner.user.identityId);                                                   // 395
    }                                                                                                 // 396
  }                                                                                                   // 397
                                                                                                      // 398
  return result;                                                                                      // 399
}                                                                                                     // 400
                                                                                                      // 401
SandstormPermissions.createNewApiToken = function (db, provider, grainId, petname,                    // 402
                                                   roleAssignment, forSharing,                        // 403
                                                   expiresIfUnusedDuration) {                         // 404
  // Creates a new UiView API token. If `rawParentToken` is set, creates a child token.               // 405
  check(grainId, String);                                                                             // 406
  check(petname, String);                                                                             // 407
  check(roleAssignment, db.roleAssignmentPattern);                                                    // 408
  check(forSharing, Boolean);                                                                         // 409
  // Meteor bug #3877: we get null here instead of undefined when we                                  // 410
  // explicitly pass in undefined.                                                                    // 411
  check(expiresIfUnusedDuration, Match.OneOf(undefined, null, Number));                               // 412
  check(provider, Match.OneOf({identityId: String}, {rawParentToken: String}));                       // 413
                                                                                                      // 414
  var grain = db.getGrain(grainId);                                                                   // 415
  if (!grain) {                                                                                       // 416
    throw new Meteor.Error(403, "Unauthorized", "No grain found.");                                   // 417
  }                                                                                                   // 418
                                                                                                      // 419
  var token = Random.secret();                                                                        // 420
        var apiToken = {                                                                              // 421
    _id: Crypto.createHash("sha256").update(token).digest("base64"),                                  // 422
    grainId: grainId,                                                                                 // 423
    roleAssignment: roleAssignment,                                                                   // 424
    petname: petname,                                                                                 // 425
    created: new Date(),                                                                              // 426
    expires: null,                                                                                    // 427
  };                                                                                                  // 428
                                                                                                      // 429
  if (provider.rawParentToken) {                                                                      // 430
    var parentToken = Crypto.createHash("sha256").update(provider.rawParentToken).digest("base64");   // 431
    var parentApiToken = db.collections.apiTokens.findOne(                                            // 432
      {_id: parentToken, grainId: grainId, objectId: {$exists: false}});                              // 433
    if (!parentApiToken) {                                                                            // 434
      throw new Meteor.Error(403, "No such parent token found.");                                     // 435
    }                                                                                                 // 436
    identityId = parentApiToken.identityId;                                                           // 437
    if (parentApiToken.forSharing) {                                                                  // 438
      forSharing = true;                                                                              // 439
    }                                                                                                 // 440
    apiToken.parentToken = parentToken;                                                               // 441
  } else {                                                                                            // 442
    apiToken.identityId = provider.identityId;                                                        // 443
  }                                                                                                   // 444
  apiToken.forSharing = forSharing;                                                                   // 445
                                                                                                      // 446
  if (expiresIfUnusedDuration) {                                                                      // 447
    apiToken.expiresIfUnused = new Date(Date.now() + expiresIfUnusedDuration);                        // 448
  }                                                                                                   // 449
                                                                                                      // 450
  db.collections.apiTokens.insert(apiToken);                                                          // 451
                                                                                                      // 452
  var endpointUrl = Url.parse(process.env.ROOT_URL).protocol + "//" + db.makeWildcardHost("api");     // 453
  return {id: apiToken._id, token: token, endpointUrl: endpointUrl};                                  // 454
}                                                                                                     // 455
                                                                                                      // 456
// Make self-destructing tokens actually self-destruct, so they don't                                 // 457
// clutter the token list view.                                                                       // 458
SandstormPermissions.cleanupSelfDestructing = function (db) {                                         // 459
  return function () {                                                                                // 460
    var now = new Date();                                                                             // 461
    db.collections.apiTokens.remove({expiresIfUnused: {$lt: now}});                                   // 462
  }                                                                                                   // 463
}                                                                                                     // 464
                                                                                                      // 465
Meteor.methods({                                                                                      // 466
  transitiveShares: function(identityId, grainId) {                                                   // 467
    check(identityId, String);                                                                        // 468
    check(grainId, String);                                                                           // 469
    if (this.userId) {                                                                                // 470
      var db = this.connection.sandstormDb;                                                           // 471
      return SandstormPermissions.downstreamTokens(db,                                                // 472
          {grain: {_id: grainId, identityId: identityId }});                                          // 473
    }                                                                                                 // 474
  },                                                                                                  // 475
                                                                                                      // 476
  newApiToken: function (provider, grainId, petname, roleAssignment, forSharing,                      // 477
                         expiresIfUnusedDuration) {                                                   // 478
    check(provider, Match.OneOf({identityId: String}, {rawParentToken: String}));                     // 479
    var db = this.connection.sandstormDb;                                                             // 480
    if (provider.identityId) {                                                                        // 481
      if (!this.userId || !db.userHasIdentity(this.userId, provider.identityId)) {                    // 482
        throw new Meteor.Error(403, "Not an identity of the current user: " + provider.identityId);   // 483
      }                                                                                               // 484
    }                                                                                                 // 485
    return SandstormPermissions.createNewApiToken(                                                    // 486
      this.connection.sandstormDb, provider, grainId, petname, roleAssignment, forSharing,            // 487
      expiresIfUnusedDuration);                                                                       // 488
  },                                                                                                  // 489
                                                                                                      // 490
  updateApiToken: function (token, newFields) {                                                       // 491
    var db = this.connection.sandstormDb;                                                             // 492
                                                                                                      // 493
    check(token, String);                                                                             // 494
    check(newFields, {petname: Match.Optional(String),                                                // 495
                      roleAssignment: Match.Optional(db.roleAssignmentPattern),                       // 496
                      revoked: Match.Optional(Boolean)});                                             // 497
                                                                                                      // 498
    if (!this.userId) {                                                                               // 499
      throw new Meteor.Error(403, "Must be logged in to modify a token");                             // 500
    }                                                                                                 // 501
    var apiToken = db.collections.apiTokens.findOne(token);                                           // 502
    if (!apiToken) {                                                                                  // 503
      throw new Meteor.Error(404, "No such token found.");                                            // 504
    }                                                                                                 // 505
                                                                                                      // 506
    if (_.findWhere(SandstormDb.getUserIdentities(db.getUser(this.userId)),                           // 507
                    {id: apiToken.identityId})) {                                                     // 508
      var modifier = {$set: newFields};                                                               // 509
      db.collections.apiTokens.update(token, modifier);                                               // 510
    } else {                                                                                          // 511
      throw new Meteor.Error(403, "User not authorized to modify this token.");                       // 512
    }                                                                                                 // 513
  }                                                                                                   // 514
});                                                                                                   // 515
                                                                                                      // 516
////////////////////////////////////////////////////////////////////////////////////////////////////////

}).call(this);


/* Exports */
if (typeof Package === 'undefined') Package = {};
Package['sandstorm-permissions'] = {
  SandstormPermissions: SandstormPermissions
};

})();

//# sourceMappingURL=sandstorm-permissions.js.map
